/*
	CS148 Starter code for assigment 8
	Note that assignment 8 does not require this file to be changed.
*/


#include <stdlib.h>
#include <math.h>

// May need to change this include to customize to your environment
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif


#include "st.h"
#include "subdiv.h"


// Current size of the OpenGL window
int windowSizeX;
int windowSizeY;

// Mouse controls
float zoom = 3.0f;
float rotx = 0;
float roty = 0.001f;
float tx = 0;
float ty = 0;
int lastx=0;
int lasty=0;
unsigned char Buttons[3] = {0};


// The scene graph
STScene scene;
SubdivSurface subdiv;
bool wireframe = true;


// number of subdivision steps that should be performed on the model
int nLevels = 0;


static void display() {

	glClearColor(.1f, .1f, .1f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	//gluLookAt(0., 0., 4.f, 0., 0., 0., 0., 1., 0.);

	glTranslatef(0,0,-zoom);
	glTranslatef(tx,ty,0);
	glRotatef(rotx,1,0,0);
	glRotatef(roty,0,1,0);	

	scene.Draw();

	glutSwapBuffers();
}


void reshape(int w, int h) {
	windowSizeX = w;
	windowSizeY = h;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	gluPerspective(60., double(windowSizeX) / double(windowSizeY), .01, 1000.);
	glViewport(0, 0, w, h);

	glMatrixMode(GL_MODELVIEW);
}


// when mouse buttons are down this handler with compute UI rotations, zooms, ect.
void handleMotion(int x,int y)
{
	int diffx=x-lastx;
	int diffy=y-lasty;
	lastx=x;
	lasty=y;

	if( Buttons[2])
	{
		zoom -= (float) 0.05f * diffx;
	}
	else
		if( Buttons[0] )
		{
			rotx += (float) 0.5f * diffy;
			roty += (float) 0.5f * diffx;		
		}
		else
			if( Buttons[1] )
			{
				tx += (float) 0.05f * diffx;
				ty -= (float) 0.05f * diffy;
			}
			glutPostRedisplay();
}


// handles mouse clicks
void handleMouse(int b,int s,int x,int y)
{
	lastx=x;
	lasty=y;
	switch(b)
	{
	case GLUT_LEFT_BUTTON:
		Buttons[0] = ((GLUT_DOWN==s)?1:0);
		break;
	case GLUT_MIDDLE_BUTTON:
		Buttons[1] = ((GLUT_DOWN==s)?1:0);
		break;
	case GLUT_RIGHT_BUTTON:
		Buttons[2] = ((GLUT_DOWN==s)?1:0);
		break;
	default:
		break;		
	}
	glutPostRedisplay();
}



static void keyboard(unsigned char key, int x, int y) {

	switch (key) {

		case char(27):
		case 'q':
		case 'Q':
			exit(0);
		
		// increase number of subdivision levels
		case '+':
		case '=':
			nLevels++;
			subdiv.ComputeSurface(nLevels);
			printf("Subdivision levels = %d\n", nLevels);
			break;

		// decrease number of subdivision levels
		case '-':
		case '_':

			if (nLevels > 0) {
				nLevels--;
				subdiv.ComputeSurface(nLevels);
				printf("Subdivision levels = %d\n", nLevels);
			}
			break;

		case 's':
		case 'S':
			wireframe = !wireframe;
			subdiv.DrawWireframe( wireframe );
			break;
	}

	glutPostRedisplay();
}


/*
	Sets up the scene 
*/
static void setup(const char* filename) {
	
	subdiv.LoadGeometry(filename);
	subdiv.ComputeSurface(nLevels);
	subdiv.SetColor(STColor(1.0f, .2f, .2f));
	subdiv.DrawWireframe(true);

	scene.GetRoot()->PlaceObj(&subdiv);
	
	glEnable(GL_DEPTH_TEST);

}



void usage(char* argv0) {
	fprintf(stderr, "Usage: %s modelname.geo\n", argv0);
}


int main(int argc, char** argv) {

	std::string objFilename;

	if (argc < 2) {
		usage(argv[0]);
		exit(1);
	}

	objFilename = std::string( argv[1] );

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowPosition(20,20);
	glutInitWindowSize(512, 512);
	glutCreateWindow("CS148 - Assignment 8");

	setup(objFilename.c_str());

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(handleMouse);
	glutMotionFunc(handleMotion);
	
	glutMainLoop();

	return 0;
}
