
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <vector>
#include <set>
#include <map>

#include "subdiv.h"
#include "stgl.h"



// STEP 1:
// CS148 TODO: This method should populate the positions array with 
// vectors corresponding to the positions of all vertices adjcent
// to this vertex
void SubdivVertex::GetAdjVertexPositions(std::vector<STVector>& positions) {

	positions.clear();

	// ADD CODE HERE!!!
	// find all the positions of vertices that are adjacent to this vertex
	// and stick them in 'positions'.  At the end of this call, positions.size()
	// should equal valence of this vertex
	// 
	// Tip:  To append to the end of a vector, use: positions.push_back( STVector(1,1,1) )
}


SubdivSurface::SubdivSurface() { }


SubdivSurface::~SubdivSurface() {

	for (int i=0; i<(int)mOrigVerts.size(); i++)
		delete mOrigVerts[i];				

	for (int i=0; i<(int)mOrigFaces.size(); i++)
		delete mOrigFaces[i];

	if (mSubdivVerts.size() != mOrigVerts.size()) {
		for (int i=0; i<(int)mSubdivVerts.size(); i++)
			delete mSubdivVerts[i];				

		for (int i=0; i<(int)mSubdivFaces.size(); i++)
			delete mSubdivFaces[i];
	}
}



// Parses a geometry file and produces a list of vertex positions and face
// vertex indices.
void SubdivSurface::LoadGeometry(const char* filename) {


	std::vector<int> indices;
	std::vector<STPoint3D> verts;


	FILE* objFile = fopen(filename, "r");
	char  token[80];

	if (!objFile) {
		fprintf(stderr, "Could not open object geometry file '%s'.\n", filename);
		exit(1);
	}

	
	// search for the start of the face indices
	while (!feof(objFile) && fscanf(objFile, "%s", token) != 1);

	if ( !(strcmp(token, "F")==0 || strcmp(token, "f")==0) ) {
		fprintf(stderr, "Unexpected file format!\n");
		exit(1);
	}
	
	// read all face indices
	while (!feof(objFile)) {

		fscanf(objFile, "%s", token);

		if ( strcmp(token, "V")==0 || strcmp(token, "v")==0 ) {
			break;
		} else {
			int idx = atoi(token);
			indices.push_back(idx);
			int size = (int)indices.size();
			if (size % 3 == 0)	 {		
				//printf("face: %d %d %d\n", indices[size-3], indices[size-2], indices[size-1]);
			}
		}
	}
	
	assert(indices.size() % 3 == 0);

	// read all vertex positions
	int count = 0;
	STPoint3D pt;
	while (!feof(objFile)) {

		fscanf(objFile, "%s", token);

		float f = (float)atof(token);
		switch (count % 3) {
			case 0:
				pt.x = f;
				break;
			case 1:
				pt.y = f;
				break;
			case 2:
				pt.z = f;
				verts.push_back(pt);
				//printf("point: %f %f %f\n", pt.x, pt.y, pt.z);
				break;
		}
		count++;
	}

	printf("The base object has %d faces containing %d vertices.\n", indices.size() / 3, verts.size());
	
	fclose(objFile);

    ProcessBaseGeometry(indices, verts);
}


/*
  Processes the geometry information (faces and vertices) read from disk
  and builds a structure that contains adjacency information
*/
void SubdivSurface::ProcessBaseGeometry(const std::vector<int>& indices, const std::vector<STPoint3D>& verts) {

	int numFaces = (int)indices.size() / 3;

	mOrigFaces.resize(numFaces);
	mOrigVerts.resize(verts.size());

	// just copy over the vertex data
	for (int i=0; i<(int)mOrigVerts.size(); i++) {
		mOrigVerts[i] = new SubdivVertex();
		mOrigVerts[i]->pos = verts[i];
		mOrigVerts[i]->idx = i;
	}

	
	std::set<SubdivEdge> knownEdges;
	

	// build the topology for the base (unsubdivided mesh)
	for (int i=0; i<(int)numFaces; i++) {

		mOrigFaces[i] = new SubdivFace;
		mOrigFaces[i]->idx = i;
		SubdivFace* curFace = mOrigFaces[i];

		// make face point to its three vertices
		curFace->verts[0] = mOrigVerts[indices[3 * i]];
		curFace->verts[1] = mOrigVerts[indices[3 * i + 1]];
		curFace->verts[2] = mOrigVerts[indices[3 * i + 2]];

		// make vertices used by this face point to face
		curFace->verts[0]->adjFace = curFace;
		curFace->verts[1]->adjFace = curFace;
		curFace->verts[2]->adjFace = curFace;


		// now find the 3 faces that are the neighboring faces of the current face
		for (int edgeNum=0; edgeNum<3; edgeNum++) {

			SubdivEdge edge( curFace->verts[edgeNum], curFace->verts[(edgeNum+1) % 3]);

			// if we haven't yet encountered the face on the other side of the edge,
			// remember: (1) the edge, (2) the current face (on this side of the edge),
			// and (3) the edge number of the edge in relation to the current face
			if (knownEdges.find(edge) == knownEdges.end()) {

				// edge doesn't exist.  Haven't encountered the edge in the past, and
				// therefore have not yet seen the face on the other side of the edge
				edge.adjFace[0] = curFace;
				edge.adjFace0EdgeNum = edgeNum;
				knownEdges.insert(edge);

			// We know the face on the other side of the edge (because we've seen
			// it before and added the edge information into the set)!  Make the
			// current face point to the face on the other side of the edge, and make
			// the other face point to the current face.  At this point we are
			// certain that no other faces can touch this edge (there can at most be 2),
			// so we just delete the edge.
			} else {
				
				SubdivEdge existingEdge = *knownEdges.find(edge);				

				curFace->adjFaces[edgeNum] = existingEdge.adjFace[0];
				existingEdge.adjFace[0]->adjFaces[existingEdge.adjFace0EdgeNum] = curFace;
				knownEdges.erase(edge);
			}
		}
	}


	// finally compute the valence of all vertices.  Instead of storing
	// the valence we just store whether or not the vertex is
	// "extrordinary" (that is, it does not have valence 6)
	for (int i=0; i<(int)mOrigVerts.size(); i++) {
		
		SubdivVertex* curVert = mOrigVerts[i];

		// Computes the valence of the vertex (how many other vertices it is connected
		// to by counting how many faces it is shared by.  This we assuming the surface
		// is manifold counting the adjacent faces is equivalent to counting the
		// vertices.
		int valence = 1;
		SubdivFace* curFace = curVert->adjFace->NextFace(curVert);
		while (curFace != NULL && curFace != curVert->adjFace) {
			valence++;
			curFace = curFace->NextFace(curVert);
		}

		if (curFace == NULL) {
			fprintf(stderr,
				    "vertex: %d -- Uh oh! Surface has a boundary! Not handling this case in assignment 8!\n",
					curVert->idx);
			exit(1);
		}

		if (valence != 6)
			curVert->extraordinary = true;
	}

	// debug
	//for (int i=0; i<(int)numFaces; i++) {
	//	SubdivFace* curFace = mOrigFaces[i];
	//	printf("face %d:\n", i);
	//	printf("    verts: %d %d %d\n", curFace->verts[0]->idx, curFace->verts[1]->idx, curFace->verts[2]->idx);
	//	printf("    faces: %d %d %d\n", curFace->adjFaces[0]->idx, curFace->adjFaces[1]->idx, curFace->adjFaces[2]->idx);
	//}
	
}


/*
   Returns a new position for an EVEN vertex given the parent vertex 'vert',
   the surrounding vertex positions, and a value for beta.
 */
STPoint3D SubdivSurface::PositionFromOneRing(SubdivVertex* vert, const std::vector<STVector>& adjPositions, float beta) {
	
	// CS148 TODO: compute a new position given 'vert' and all the
	// positions of it's surrounding vertices.  The result vertex should be
	// a linear combination of all adjacent vertices (which are weighted by beta) and the
	// parent vertex, which is weighted by (1 - beta * it's valence).

	return STPoint3D(0.0f ,0.0f, 0.0f);
}


/*
	Performs nLevels levels of loop subdivision on the base surface.
	After subdividing the geometry, surface normals are computed
*/
void SubdivSurface::ComputeSurface(int nLevels) {

	mSubdivVerts = mOrigVerts;
	mSubdivFaces = mOrigFaces;


	printf("Subdivision not implemented yet!!! Returning immediately from ComputeSurface()\n");
	return;


	std::vector<STVector> adjPositions;
	
	//	For each level i, we subdivide the mesh produced in level i-1
	//	In the first iteration, we subdivide the "base" mesh that was
	//	read from the data file on disk.

	for (int level=0; level<nLevels; level++) {

		// You will need to add the vertices and faces produced in this level
		// of subdivision to these vertors
		std::vector<SubdivVertex*> newVerts;
		std::vector<SubdivFace*> newFaces;


		//     Allocate new faces produced in this subdivision
		//   * Each face should be subdivided into four child faces.
		//   * Pointers to the 4 newly allocated faces are stored in
		//     the "childFaces[]" array of the current face
		for (int i = 0; i < (int)mSubdivFaces.size(); i++) {
			for (int j=0; j<4; j++) {
				mSubdivFaces[i]->childFaces[j] = new SubdivFace;
				newFaces.push_back(mSubdivFaces[i]->childFaces[j]);
			}
		}



		// STEP 2.  All the vertices present in the mesh at the last 
		// subdivision level are also present in the new subdivided mesh.
		// We call these vertices the EVEN vertices in the new mesh
		for (int i=0; i < (int)mSubdivVerts.size(); i++) {

	
			SubdivVertex* vert = mSubdivVerts[i];

			// create an EVEN vertex in the subdivided mesh that corresponds to
			// a vertex ('vert') in the mesh from the previous level of subdivision.  The
			// EVEN vertex in the new mesh is a child of the corresponding vertex in the
			// previous mesh
			vert->childVert = new SubdivVertex;
			vert->childVert->extraordinary = vert->extraordinary;			
			newVerts.push_back(vert->childVert);


			// CS148 TODO: 
			// The EVEN vertex in the subdivided mesh now needs to be moved to a new position.
			// This position is computed from the position of it's parent vertex and the vertices
			// adjacent to the parent vertex in the previous mesh.
			// Compute the new position of the EVEN vertex here!
			// Note: recall that beta can be set directly to 1/16 if the parent is NOT an
			// extrordinary vertex

			// ADD CODE HERE!



			// CS148 TODO:
			// The vertex in the subdivided mesh needs to point to one of the faces
			// in the mesh.  Please set this pointer here!
			
			// ADD CODE HERE! 
			
		}


		// STEP 3: Now you will need to create a new vertex for each edge in the
		// old mesh.  The new vertices are called ODD vertices.
		std::map<SubdivEdge, SubdivVertex*> edgeVerts;

		for (int i=0; i<(int)mSubdivFaces.size(); i++) {
			SubdivFace *curFace = mSubdivFaces[i];

			for (int j=0; j<3; j++) {

				// CS148 TODO: Compute new vertex on jth edge of the current face here
				//      * 1. Determine if the vertex on this edge has already been created
				//        (note that you will see each edge twice when iterating over the faces)
				//      * 2. If it needs to be created, create a new vertex and compute it's
				//           position

				SubdivEdge edge(curFace->verts[j], curFace->verts[(j+1) % 3]);
				SubdivVertex *vert = edgeVerts[edge];

				// ADD CODE HERE
				// 'vert' will be NULL if edge is NOT in the map
				// If edge is NOT in the map
					// 1. create and initialize a new odd vertex since
				    // 2. compute the position of the new odd vertex
				    // 3. place the vertex in the edgeVerts map, indiced by 'edge'
				    //       (edgeVerts[edge] = vert;
			}
		}		


		// STEP 4: Build topology.  For each new face in the subdivided mesh,
		// please set it's adjacent faces here.		
		for (int i=0; i<(int)mSubdivFaces.size(); i++) {

			SubdivFace* curFace = mSubdivFaces[i];
			
			// CS148 TODO: 
			// For each child face (curFace->childFaces[j]), you'll need to appropriately
			// set the adjacent face pointers (curFace->childFaces[j]->adjFaces[k])

			// ADD CODE HERE

		}


		// STEP 5: Update face vertex pointers for all the new faces
		for (int i=0; i<(int)mSubdivFaces.size(); i++) {

			SubdivFace* curFace = mSubdivFaces[i];

			// CS148 TODO:
			// For each child face (curFace->childFaces[j]), you'll need to appropriately
			// set the face's vertex pointers (curFace->childFaces[j]->verts[k])

			// ADD CODE HERE
		}


		// cleanup the previous subdivision level geometry so we don't leak memory
		if (level != 0) {

			for (int i=0; i<(int)mSubdivVerts.size(); i++)
				delete mSubdivVerts[i];				

			for (int i=0; i<(int)mSubdivFaces.size(); i++)
				delete mSubdivFaces[i];
		}

		mSubdivVerts = newVerts;
		mSubdivFaces = newFaces;

	}


	printf("%d levels of subdivision: %d total faces, %d total vertices.\n",
		nLevels, mSubdivFaces.size(), mSubdivVerts.size());


	if (nLevels > 0) {

		// If we've performed subdivision, we now want to push the final vertices
		// out toward the limit surface.  This step was not discussed in lecture.  
		// So will implement it for you here.

		// push the vertices resulting from subdivision out to the limit surface
		STPoint3D* limitPos = new STPoint3D[mSubdivVerts.size()];

		for (int i=0; i<(int)mSubdivVerts.size(); i++) {

			SubdivVertex* vert = mSubdivVerts[i];
			vert->GetAdjVertexPositions(adjPositions);

			int valence = (int)adjPositions.size();
			float beta = gamma(valence);

			limitPos[i] = PositionFromOneRing(vert, adjPositions, beta);
		}

		for (int i=0; i<(int)mSubdivVerts.size(); i++)
			mSubdivVerts[i]->pos = limitPos[i];

		delete [] limitPos;		


		// Lastly, we need to compute normals for shading
		mSubdivNormals.resize(mSubdivVerts.size());

		for (int i=0; i<(int)mSubdivVerts.size(); i++) {
				
			STVector s;
			STVector t;

			mSubdivVerts[i]->normal = &mSubdivNormals[i];

			mSubdivVerts[i]->GetAdjVertexPositions(adjPositions);

			int valence = (int)adjPositions.size();
			float val = 2.0f * (float)M_PI / valence;

			for (int j=0; j<valence; j++) {
				s += cosf(val * j) * adjPositions[j];
				t += sinf(val * j) * adjPositions[j];
			}


			// CS148 TODO.  The code above has produce two vectors TANGENT to the
			// vertex.  's' and 't'.  There is a very simple way to compute a normal 
			// vector to the surface using 's' and 't'.  
			
			// ADD CODE HERE! to set the value of the STVector pointed to by mSubdivVerts[i]->normal
		}

	}
}


/*
	Iterates over the faces of the finest subdivision mesh and makes
	OpenGL calls to draw the mesh.  Depending on the current drawwing
	mode the mesh will be drawn as a smooth shaded shiny solid or as a
	plain wireframe mesh.
*/
void SubdivSurface::Draw() {


	if (mDrawWireframe || mSubdivFaces[0]->verts[0]->normal == NULL) {

		if (glIsEnabled(GL_LIGHTING))
			glDisable(GL_LIGHTING);

		glColor3f(mColor.r, mColor.g, mColor.b);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

		glBegin(GL_TRIANGLES);

		for (int i=0; i<(int)mSubdivFaces.size(); i++) {
			SubdivFace* curFace = mSubdivFaces[i];

			//if (curFace->boundary)
			//	glColor3f(0.0f, 1.0f, 0.0f);
			//else
			glColor3f(mColor.r, mColor.g, mColor.b);

			for (int j=0; j<3; j++) {
				SubdivVertex* vert = curFace->verts[j];
				glVertex3f(vert->pos.x, vert->pos.y, vert->pos.z);
			}
		}

		glEnd();


	} else {

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		float light0_ambient[] = {1.0f, 1.0f, 1.0f, 1.0f};
		float light0_diffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};
		float light0_specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
		float light0_position[] = {1.0f, 1.0f, 1.0f, 0.0f};

	    glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
	    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	    glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
	    glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

		float red[] = {mColor.r, mColor.g, mColor.b, 1.0f};
		float white[] = {1.0f, 1.0f, 1.0f, 1.0f};
		float ambient[] = {0.11f, 0.11f, 0.11f, 1.0f};
		float shiney[] = {100.0f};
		
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, red);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shiney);

		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		// set material and light parameters here

		glBegin(GL_TRIANGLES);

		for (int i=0; i<(int)mSubdivFaces.size(); i++) {
			SubdivFace* curFace = mSubdivFaces[i];
			for (int j=0; j<3; j++) {
				SubdivVertex* vert = curFace->verts[j];
				glNormal3f(vert->normal->x, vert->normal->y, vert->normal->z);				
				glVertex3f(vert->pos.x, vert->pos.y, vert->pos.z);
			}
		}

		glEnd();

	}


	glEnd();
}
