#include "STColor.h"
#include "STImage.h"
#include <math.h>

#ifndef max
#define max(a,b)     (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)     (((a) < (b)) ? (a) : (b))
#endif

/* Constructors */
STColor::STColor() 
: r(0.f), g(0.f), b(0.f)
{
}

STColor::STColor(float _c)
: r(_c), g(_c), b(_c)
{
}

STColor::STColor(float _r, float _g, float _b) 
: r(_r), g(_g), b(_b)
{
}

STColor::STColor(const STColor& c) 
: r(c.r), g(c.g), b(c.b)
{
}

STColor::STColor(const STPixel& pix) 
{
	this->r = pix.r / 255.f;
	this->g = pix.g / 255.f;
	this->b = pix.b / 255.f;
}

/* Math operators */
STColor STColor::operator+(const STColor& rhs) const {
	STColor rval;
	rval.r = this->r + rhs.r;
	rval.g = this->g + rhs.g;
	rval.b = this->b + rhs.b;
	return rval;
}
STColor& STColor::operator+=(const STColor& rhs) {
	this->r += rhs.r;
	this->g += rhs.g;
	this->b += rhs.b;
	return *this;
}
STColor STColor::operator-(const STColor& rhs) const {
	STColor rval;
	rval.r = this->r - rhs.r;
	rval.g = this->g - rhs.g;
	rval.b = this->b - rhs.b;
	return rval;
}
STColor& STColor::operator-=(const STColor& rhs) {
	this->r -= rhs.r;
	this->g -= rhs.g;
	this->b -= rhs.b;
	return *this;
}
STColor STColor::operator*(const float s) const {
	STColor rval;
	rval.r = this->r * s;
	rval.g = this->g * s;
	rval.b = this->b * s;
	return rval;
}
STColor STColor::operator*(const STColor& s) const {
	STColor rval;
	rval.r = this->r * s.r;
	rval.g = this->g * s.g;
	rval.b = this->b * s.b;
	return rval;
}
STColor& STColor::operator*=(const float s) {
	this->r *= s;
	this->b *= s;
	this->g *= s;
	return *this;
}
STColor& STColor::operator*=(const STColor& s) {
	this->r *= s.r;
	this->b *= s.g;
	this->g *= s.b;
	return *this;
}
STColor STColor::operator/(const float s) const {
	return *this * (1.f / s);
}
STColor STColor::operator/(const STColor& s) const {
	STColor result;
	result.r = this->r / s.r;
	result.g = this->g / s.g;
	result.b = this->b / s.b;
	return result;
}
STColor& STColor::operator/=(const float s) {
	this->r /= s;
	this->g /= s;
	this->b /= s;
	return *this;
}
STColor& STColor::operator/=(const STColor& s) {
	this->r /= s.r;
	this->g /= s.g;
	this->b /= s.b;
	return *this;
}

/* Extra operator to allow float*Color expression */
STColor operator*(float lhs, STColor &rhs) {
	// just flip it around
	return rhs * lhs;
}

/* Component-wise exponential */
STColor STColor::Exp() const {
	STColor result;
	result.r = expf(this->r);
	result.g = expf(this->g);
	result.b = expf(this->b);
	return result;
}
/* Component-wise log */
STColor STColor::Log() const {
	STColor result;
	result.r = logf(this->r);
	result.g = logf(this->g);
	result.b = logf(this->b);
	return result;
}

/* Luminance */
float STColor::Y() const {
	return .2126f*this->r + .7152f*this->g + .0722f*this->b;
}

/* Conversion */
STPixel STColor::ToPixel() const {
	STPixel result;
	result.r = (unsigned char) max(0.f, min(255.f, (r * 255.f)));
	result.g = (unsigned char) max(0.f, min(255.f, (g * 255.f)));
	result.b = (unsigned char) max(0.f, min(255.f, (b * 255.f)));
	result.a = 255;
	return result;
}
