#include "STTexture.h"
#include "stgl.h"

#define STTEXTURE_DEFAULT_FILTER_MIN GL_LINEAR
#define STTEXTURE_DEFAULT_FILTER_MAG GL_LINEAR
#define STTEXTURE_DEFAULT_WRAP_S GL_CLAMP
#define STTEXTURE_DEFAULT_WRAP_T GL_CLAMP

/* Constructor, creates the  GL texture and
 * sets some default texture parameters.
 */
STTexture::STTexture() {
	glGenTextures(1, (GLuint*)&texid);
	SetFilter(	STTEXTURE_DEFAULT_FILTER_MAG,
				STTEXTURE_DEFAULT_FILTER_MIN );
	SetWrap(	STTEXTURE_DEFAULT_WRAP_S,
				STTEXTURE_DEFAULT_WRAP_T );
}

/* Destructor, destroys the GL texture. */
STTexture::~STTexture() {
	glDeleteTextures(1, (const GLuint*)&texid);
}

/* Upload an STImage to the texture.  For
 * older versions of GL this may require 
 * the width and height to be powers of 2.
 * This code is extremely simple because
 * STImage guarantees the format of the
 * image data and we simply use the same
 * format.
 */
void STTexture::Upload(STImage& teximage, bool mipmap) {
	Bind();

	if (mipmap) {
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA,
						  teximage.GetWidth(), teximage.GetHeight(),
						  GL_RGBA, GL_UNSIGNED_BYTE, teximage.Pixels());
	}
	else {
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
					 teximage.GetWidth(), teximage.GetHeight(), 0,
					 GL_RGBA, GL_UNSIGNED_BYTE, teximage.Pixels());
	}
	Release();
}

/* Bind the texture as the current texture. */
void STTexture::Bind() {
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texid);
}

/* Release the texture. */
void STTexture::Release() {
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
}

/* Set filter parameters. */
void STTexture::SetFilter(int fmag, int fmin) {
	Bind();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, fmag);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, fmin);
	Release();
}

/* Set wrap parameters. */
void STTexture::SetWrap(int ws, int wt) {
	Bind();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, ws);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wt);
	Release();
}
