
#include <math.h>
#include "STVector.h"


/* Math operators */
STVector STVector::operator+(const STVector& rhs) const {
	STVector rval;
	rval.x = this->x + rhs.x;
	rval.y = this->y + rhs.y;
	rval.z = this->z + rhs.z;
	return rval;
}
STVector& STVector::operator+=(const STVector& rhs) {
	this->x += rhs.x;
	this->y += rhs.y;
	this->z += rhs.z;
	return *this;
}
STVector STVector::operator-(const STVector& rhs) const {
	STVector rval;
	rval.x = this->x - rhs.x;
	rval.y = this->y - rhs.y;
	rval.z = this->z - rhs.z;
	return rval;
}
STVector& STVector::operator-=(const STVector& rhs) {
	this->x -= rhs.x;
	this->y -= rhs.y;
	this->z -= rhs.z;
	return *this;
}
STVector STVector::operator*(const float s) const {
	STVector rval;
	rval.x = this->x * s;
	rval.y = this->y * s;
	rval.z = this->z * s;
	return rval;
}
STVector STVector::operator*(const STVector& s) const {
	STVector rval;
	rval.x = this->x * s.x;
	rval.y = this->y * s.y;
	rval.z = this->z * s.z;
	return rval;
}
STVector& STVector::operator*=(const float s) {
	this->x *= s;
	this->z *= s;
	this->y *= s;
	return *this;
}
STVector& STVector::operator*=(const STVector& s) {
	this->x *= s.x;
	this->z *= s.y;
	this->y *= s.z;
	return *this;
}
STVector STVector::operator/(const float s) const {
	return *this * (1.f / s);
}
STVector STVector::operator/(const STVector& s) const {
	STVector result;
	result.x = this->x / s.x;
	result.y = this->y / s.y;
	result.z = this->z / s.z;
	return result;
}
STVector& STVector::operator/=(const float s) {
	this->x /= s;
	this->y /= s;
	this->z /= s;
	return *this;
}
STVector& STVector::operator/=(const STVector& s) {
	this->x /= s.x;
	this->y /= s.y;
	this->z /= s.z;
	return *this;
}

float STVector::Length() const {
	return sqrt( Dot(*this, *this) );
}

/* Extra operator to allow float*Color expression */
STVector operator*(float lhs, const STVector &rhs) {
	// just flip it around
	return rhs * lhs;
}

STVector Cross(const STVector& v1, const STVector& v2) {
	STVector v;
	v.x = v1.y * v2.z - v1.z * v2.y;
	v.y = v1.z * v2.x - v1.x * v2.z;
	v.z = v1.x * v2.y - v1.y * v2.x;
	return v;
}

float Dot(const STVector& v1, const STVector& v2) {
	return v1.x*v2.x + v1.y*v2.y + v1.z*v1.z;
}

STVector Normalize(const STVector& v) {
	float scale = 1.0f / sqrt(Dot(v, v));
	return scale * v;
}