
#ifndef __STCOLOR_H__
#define __STCOLOR_H__

// We need this but including st.h causes loops
class STPixel;

/**
*  Simple class representing a color with 3 floating point values
*  Only the values 0.f to 1.f are displayable, but the color can
*  contain any values.
*/
class STColor {
public:
	float r, g, b;

	/* Constructors */
	STColor();
	STColor(float _c);
	STColor(float _r, float _g, float _b);
	STColor(const STColor& c);
	STColor(const STPixel& pix);
	/* Math operators */
	STColor operator+(const STColor& rhs) const;
	STColor& operator+=(const STColor& rhs);
	STColor operator-(const STColor& rhs) const;
	STColor& operator-=(const STColor& rhs);
	STColor operator*(const float s) const;
	STColor operator*(const STColor& s) const;
	STColor& operator*=(const float s);
	STColor& operator*=(const STColor& s);
	STColor operator/(const float s) const;
	STColor operator/(const STColor& s) const;
	STColor& operator/=(const float s);
	STColor& operator/=(const STColor& s);

	/* Component-wise exponential */
	STColor Exp() const;
	/* Component-wise log */
	STColor Log() const;

	/* Luminance */
	float Y() const;

	/* Conversion */
	STPixel ToPixel() const;
};

/* Extra operator to allow float*Color expression */
STColor operator*(float lhs, const STColor &rhs);

#endif //__STCOLOR_H_

