
#ifndef __STFONTFACE_H__
#define __STFONTFACE_H__

#include <vector>
#include <map>

// freetype2 headers
#include "ft2build.h"
#include FT_FREETYPE_H


/**
* A font engine. An instance of STFontFace encapsulates a face of a certain
* size.  This class, which wraps Freetype2 functionality, provides information
* about the face and manages the rendering of text strings to the screen using
* this face.  Rendering is performed via bitmap glyphs. The functionality in
* this class is used heavily by STText.  Users should pay attention to units
* when using STFontFace.
*/
class STFontFace {
	
	public:

	STFontFace(const char* fontName, int fontSize);
	~STFontFace();

	// Units of font size are points
	void  SetSize(int fontSize);
	int   GetSize() const;
	void  SetFace(const char* fontName, int fontSize);

	// Return width of rendered string in units of pixels
	float DrawString(const char* str);
	float ComputeWidth(const char* str);

	// The following accessors return values in units of pixels
	// They are floating point values to provide subpixel precision
	float GetHeight() const;
	float GetAscender() const;
	float GetDescender() const;

	private:

		// Simple struct used to encapsulate all the necessary information about 
		// a bitmap glyph stored in the format expect by glBitmap
		class STBitmapGlyph {
			public:
				unsigned char* data;

				// all the following fields are in units of pixels.  advanceX is
				// of type float to provide subpixel precision when manipulating
				// the OpenGL raster position when drawing font glyphs

				int width;        // width of bitmap
				int height;       // height of bitmap
				int pitch;        // bytes between start of bitmap rows
				int offsetX;      // offset of origin of bitmap from origin in coordinate system
				int offsetY;      
				float advanceX;   // pixel advancement to next character
		};

		int GetBitmapIndex(char character);
		int GenerateBitmap(char character);

		static FT_Library* sFTLibrary;
		static const int kDefaultFontSize = 12;
			
		FT_Face   mFTFace;
		int		  mSize; // in points

		std::map<char,int> mCharMap;
		std::vector<STBitmapGlyph> mGlyphBitmaps;

};


#endif // __STFONTFACE_H__

